@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (1, 3, 9)
-CODENAME = "god dag"
-BUILD_DT = (2022, 8, 4)
+VERSION = (1, 4, 1)
+CODENAME = "mostly reliable"
+BUILD_DT = (2022, 9, 24)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
