@@ -24,7 +24,12 @@ try:
 except:
     pass
 
-from .__init__ import ANYWIN, PY2, TYPE_CHECKING, E, unicode
+try:
+    from ipaddress import IPv6Address
+except:
+    pass
+
+from .__init__ import ANYWIN, PY2, TYPE_CHECKING, EnvParams, unicode
 from .authsrv import VFS  # typechk
 from .bos import bos
 from .star import StreamTar
@@ -69,6 +74,7 @@ from .util import (
     unescape_cookie,
     unquote,
     unquotep,
+    vjoin,
     vol_san,
     vsplit,
     yieldfile,
@@ -103,12 +109,14 @@ class HttpCli(object):
         self.ip = conn.addr[0]
         self.addr   = conn.addr
         self.args = conn.args  # mypy404
+        self.E  = self.args.E
         self.asrv = conn.asrv  # mypy404
         self.ico = conn.ico  # mypy404
         self.thumbcli = conn.thumbcli  # mypy404
         self.u2fh = conn.u2fh  # mypy404
         self.log_func = conn.log_func  # mypy404
         self.log_src = conn.log_src  # mypy404
+        self.bans = conn.hsrv.bans
         self.gen_fk = self._gen_fk if self.args.log_fk else gen_filekey
         self.tls  = hasattr(self.s, "cipher")
 
@@ -122,7 +130,6 @@ class HttpCli(object):
         self.ua = " "
         self.is_rclone = False
         self.is_ancient = False
-        self.dip = " "
         self.ouparam   = {}
         self.uparam   = {}
         self.cookies   = {}
@@ -263,7 +270,20 @@ class HttpCli(object):
 
                 self.log_src = self.conn.set_rproxy(self.ip)
 
-        self.dip = self.ip.replace(":", ".")
+        if self.bans:
+            ip = self.ip
+            if ":" in ip and not PY2:
+                ip = IPv6Address(ip).exploded[:-20]
+
+            if ip in self.bans:
+                ban = self.bans[ip] - time.time()
+                if ban < 0:
+                    self.log("client unbanned", 3)
+                    del self.bans[ip]
+                else:
+                    self.log("banned for {:.0f} sec".format(ban), 6)
+                    self.reply(b"thank you for playing", 403)
+                    return False
 
         if self.args.ihead:
             keys = self.args.ihead
@@ -402,13 +422,19 @@ class HttpCli(object):
             except Pebkac:
                 return False
 
+    def dip(self)  :
+        if self.args.plain_ip:
+            return self.ip.replace(":", ".")
+        else:
+            return self.conn.iphash.s(self.ip)
+
     def permit_caching(self)  :
         cache = self.uparam.get("cache")
         if cache is None:
             self.out_headers.update(NO_CACHE)
             return
 
-        n = "604800" if cache == "i" else cache or "69"
+        n = "604869" if cache == "i" else cache or "69"
         self.out_headers["Cache-Control"] = "max-age=" + n
 
     def k304(self)  :
@@ -461,7 +487,13 @@ class HttpCli(object):
         headers   = None,
         volsan  = False,
     )  :
-        # TODO something to reply with user-supplied values safely
+        if status == 404:
+            g = self.conn.hsrv.g404
+            if g.lim:
+                bonk, ip = g.bonk(self.ip, self.vpath)
+                if bonk:
+                    self.log("client banned: 404s", 1)
+                    self.conn.hsrv.bans[ip] = bonk
 
         if volsan:
             vols = list(self.asrv.vfs.all_vols.values())
@@ -553,7 +585,7 @@ class HttpCli(object):
             if self.vpath.startswith(".cpr/ico/"):
                 return self.tx_ico(self.vpath.split("/")[-1], exact=True)
 
-            static_path = os.path.join(E.mod, "web/", self.vpath[5:])
+            static_path = os.path.join(self.E.mod, "web/", self.vpath[5:])
             return self.tx_file(static_path)
 
         if "cf_challenge" in self.uparam:
@@ -715,6 +747,7 @@ class HttpCli(object):
         # post_sz, sha_hex, sha_b64, remains, path, url
         reader, remains = self.get_body_reader()
         vfs, rem = self.asrv.vfs.get(self.vpath, self.uname, False, True)
+        rnd, want_url, lifetime = self.upload_flags(vfs)
         lim = vfs.get_dbv(rem)[0].lim
         fdir = vfs.canonical(rem)
         if lim:
@@ -777,16 +810,22 @@ class HttpCli(object):
             else:
                 self.log("fallthrough? thats a bug", 1)
 
-        suffix = "-{:.6f}-{}".format(time.time(), self.dip)
+        suffix = "-{:.6f}-{}".format(time.time(), self.dip())
+        nameless = not fn
+        if nameless:
+            suffix += ".bin"
+            fn = "put" + suffix
+
         params = {"suffix": suffix, "fdir": fdir}
         if self.args.nw:
             params = {}
             fn = os.devnull
 
         params.update(open_ka)
+        assert fn
 
-        if not fn:
-            fn = "put" + suffix
+        if rnd and not self.args.nw:
+            fn = self.rand_name(fdir, fn, rnd)
 
         with ren_open(fn, *open_a, **params) as zfw:
             f, fn = zfw["orz"]
@@ -805,6 +844,28 @@ class HttpCli(object):
         if self.args.nw:
             return post_sz, sha_hex, sha_b64, remains, path, ""
 
+        if nameless and "magic" in vfs.flags:
+            try:
+                ext = self.conn.hsrv.magician.ext(path)
+            except Exception as ex:
+                self.log("filetype detection failed for [{}]: {}".format(path, ex), 6)
+                ext = None
+
+            if ext:
+                if rnd:
+                    fn2 = self.rand_name(fdir, "a." + ext, rnd)
+                else:
+                    fn2 = fn.rsplit(".", 1)[0] + "." + ext
+
+                params["suffix"] = suffix[:-4]
+                with ren_open(fn, *open_a, **params) as zfw:
+                    f, fn = zfw["orz"]
+
+                path2 = os.path.join(fdir, fn2)
+                atomic_move(path, path2)
+                fn = fn2
+                path = path2
+
         vfs, rem = vfs.get_dbv(rem)
         self.conn.hsrv.broker.say(
             "up2k.hash_file",
@@ -813,7 +874,7 @@ class HttpCli(object):
             rem,
             fn,
             self.ip,
-            time.time(),
+            time.time() - lifetime,
         )
 
         vsuf = ""
@@ -841,10 +902,39 @@ class HttpCli(object):
         spd = self._spd(post_sz)
         t = "{} wrote {}/{} bytes to {}  # {}"
         self.log(t.format(spd, post_sz, remains, path, sha_b64[:28]))  # 21
-        t = "{}\n{}\n{}\n{}\n".format(post_sz, sha_b64, sha_hex[:56], url)
+
+        ac = self.uparam.get(
+            "want", self.headers.get("accept", "").lower().split(";")[-1]
+        )
+        if ac == "url":
+            t = url
+        else:
+            t = "{}\n{}\n{}\n{}\n".format(post_sz, sha_b64, sha_hex[:56], url)
+
         self.reply(t.encode("utf-8"))
         return True
 
+    def rand_name(self, fdir , fn , rnd )  :
+        ok = False
+        try:
+            ext = "." + fn.rsplit(".", 1)[1]
+        except:
+            ext = ""
+
+        for extra in range(16):
+            for _ in range(16):
+                if ok:
+                    break
+
+                nc = rnd + extra
+                nb = int((6 + 6 * nc) / 8)
+                zb = os.urandom(nb)
+                zb = base64.urlsafe_b64encode(zb)
+                fn = zb[:nc].decode("utf-8") + ext
+                ok = not bos.path.exists(os.path.join(fdir, fn))
+
+        return fn
+
     def _spd(self, nbytes , add  = True)  :
         if add:
             self.conn.nbyte += nbytes
@@ -926,7 +1016,7 @@ class HttpCli(object):
         except:
             raise Pebkac(422, "you POSTed invalid json")
 
-        # self.reply(b" DD" + b"oS Protection ", 503)
+        # self.reply(b"cloudflare", 503)
         # return True
 
         if "srch" in self.uparam or "srch" in body:
@@ -1168,6 +1258,14 @@ class HttpCli(object):
             msg = "login ok"
             dur = int(60 * 60 * self.args.logout)
         else:
+            self.log("invalid password: {}".format(pwd), 3)
+            g = self.conn.hsrv.gpwd
+            if g.lim:
+                bonk, ip = g.bonk(self.ip, pwd)
+                if bonk:
+                    self.log("client banned: invalid passwords", 1)
+                    self.conn.hsrv.bans[ip] = bonk
+
             msg = "naw dude"
             pwd = "x"  # nosec
             dur = None
@@ -1242,6 +1340,22 @@ class HttpCli(object):
         self.redirect(vpath, "?edit")
         return True
 
+    def upload_flags(self, vfs )    :
+        srnd = self.uparam.get("rand", self.headers.get("rand", ""))
+        rnd = int(srnd) if srnd and not self.args.nw else 0
+        ac = self.uparam.get(
+            "want", self.headers.get("accept", "").lower().split(";")[-1]
+        )
+        want_url = ac == "url"
+        zs = self.uparam.get("life", self.headers.get("life", ""))
+        if zs:
+            vlife = vfs.flags.get("lifetime") or 0
+            lifetime = max(0, int(vlife - int(zs)))
+        else:
+            lifetime = 0
+
+        return rnd, want_url, lifetime
+
     def handle_plain_upload(self)  :
         assert self.parser
         nullwrite = self.args.nw
@@ -1257,9 +1371,12 @@ class HttpCli(object):
             if not nullwrite:
                 bos.makedirs(fdir_base)
 
+        rnd, want_url, lifetime = self.upload_flags(vfs)
+
         files       = []
         # sz, sha_hex, sha_b64, p_file, fname, abspath
         errmsg = ""
+        dip = self.dip()
         t0 = time.time()
         try:
             assert self.parser.gen
@@ -1273,10 +1390,13 @@ class HttpCli(object):
                     p_file or "", "", [".prologue.html", ".epilogue.html"]
                 )
                 if p_file and not nullwrite:
+                    if rnd:
+                        fname = self.rand_name(fdir, fname, rnd)
+
                     if not bos.path.isdir(fdir):
                         raise Pebkac(404, "that folder does not exist")
 
-                    suffix = "-{:.6f}-{}".format(time.time(), self.dip)
+                    suffix = "-{:.6f}-{}".format(time.time(), dip)
                     open_args = {"fdir": fdir, "suffix": suffix}
 
                     # reserve destination filename
@@ -1343,7 +1463,7 @@ class HttpCli(object):
                         vrem,
                         fname,
                         self.ip,
-                        time.time(),
+                        time.time() - lifetime,
                     )
                     self.conn.nbyte += sz
 
@@ -1419,21 +1539,31 @@ class HttpCli(object):
         vspd = self._spd(sz_total, False)
         self.log("{} {}".format(vspd, msg))
 
-        if not nullwrite:
-            log_fn = "up.{:.6f}.txt".format(t0)
-            with open(log_fn, "wb") as f:
-                ft = "{}:{}".format(self.ip, self.addr[1])
-                ft = "{}\n{}\n{}\n".format(ft, msg.rstrip(), errmsg)
-                f.write(ft.encode("utf-8"))
+        suf = ""
+        if not nullwrite and self.args.write_uplog:
+            try:
+                log_fn = "up.{:.6f}.txt".format(t0)
+                with open(log_fn, "wb") as f:
+                    ft = "{}:{}".format(self.ip, self.addr[1])
+                    ft = "{}\n{}\n{}\n".format(ft, msg.rstrip(), errmsg)
+                    f.write(ft.encode("utf-8"))
+            except Exception as ex:
+                suf = "\nfailed to write the upload report: {}".format(ex)
 
         sc = 400 if errmsg else 200
-        if "j" in self.uparam:
+        if want_url:
+            msg = "\n".join([x["url"] for x in jmsg["files"]])
+            if errmsg:
+                msg += "\n" + errmsg
+
+            self.reply(msg.encode("utf-8", "replace"), status=sc)
+        elif "j" in self.uparam:
             jtxt = json.dumps(jmsg, indent=2, sort_keys=True).encode("utf-8", "replace")
             self.reply(jtxt, mime="application/json", status=sc)
         else:
             self.redirect(
                 self.vpath,
-                msg=msg,
+                msg=msg + suf,
                 flavor="return to",
                 click=False,
                 status=sc,
@@ -1713,7 +1843,7 @@ class HttpCli(object):
         # send reply
 
         if is_compressed:
-            self.out_headers["Cache-Control"] = "max-age=573"
+            self.out_headers["Cache-Control"] = "max-age=604869"
         else:
             self.permit_caching()
 
@@ -1836,9 +1966,11 @@ class HttpCli(object):
         if len(ext) > 11:
             ext = "⋯" + ext[-9:]
 
-        mime, ico = self.ico.get(ext, not exact)
+        # chrome cannot handle more than ~2000 unique SVGs
+        chrome = " rv:" not in self.ua
+        mime, ico = self.ico.get(ext, not exact, chrome)
 
-        dt = datetime.utcfromtimestamp(E.t0)
+        dt = datetime.utcfromtimestamp(self.E.t0)
         lm = dt.strftime("%a, %d %b %Y %H:%M:%S GMT")
         self.reply(ico, mime=mime, headers={"Last-Modified": lm})
         return True
@@ -1851,7 +1983,7 @@ class HttpCli(object):
                 return self.tx_404(True)
 
         tpl = "mde" if "edit2" in self.uparam else "md"
-        html_path = os.path.join(E.mod, "web", "{}.html".format(tpl))
+        html_path = os.path.join(self.E.mod, "web", "{}.html".format(tpl))
         template = self.j2j(tpl)
 
         st = bos.stat(fs_path)
@@ -1866,7 +1998,7 @@ class HttpCli(object):
             for c, v in [(b"&", 4), (b"<", 3), (b">", 3)]:
                 sz_md += (len(buf) - len(buf.replace(c, b""))) * v
 
-        file_ts = max(ts_md, ts_html, E.t0)
+        file_ts = max(ts_md, ts_html, self.E.t0)
         file_lastmod, do_send = self._chk_lastmod(file_ts)
         self.out_headers["Last-Modified"] = file_lastmod
         self.out_headers.update(NO_CACHE)
@@ -2318,8 +2450,9 @@ class HttpCli(object):
 
         if not is_dir and (self.can_read or self.can_get):
             if not self.can_read and "fk" in vn.flags:
+                vabs = vjoin(vn.realpath, rem)
                 correct = self.gen_fk(
-                    self.args.fk_salt, abspath, st.st_size, 0 if ANYWIN else st.st_ino
+                    self.args.fk_salt, vabs, st.st_size, 0 if ANYWIN else st.st_ino
                 )[: vn.flags["fk"]]
                 got = self.uparam.get("k")
                 if got != correct:
@@ -2398,6 +2531,7 @@ class HttpCli(object):
             "srvinf": srv_infot,
             "acct": self.uname,
             "idx": ("e2d" in vn.flags),
+            "lifetime": vn.flags.get("lifetime") or 0,
             "perms": perms,
             "logues": logues,
             "readme": readme,
@@ -2409,6 +2543,7 @@ class HttpCli(object):
             "ls0": None,
             "acct": self.uname,
             "perms": json.dumps(perms),
+            "lifetime": ls_ret["lifetime"],
             "taglist": [],
             "def_hcols": [],
             "have_emp": self.args.emp,
@@ -2418,7 +2553,7 @@ class HttpCli(object):
             "have_mv": (not self.args.no_mv),
             "have_del": (not self.args.no_del),
             "have_zip": (not self.args.no_zip),
-            "have_unpost": (self.args.unpost > 0),
+            "have_unpost": int(self.args.unpost),
             "have_b_u": (self.can_write and self.uparam.get("b") == "u"),
             "url_suf": url_suf,
             "logues": logues,
@@ -2570,43 +2705,28 @@ class HttpCli(object):
             rd = fe["rd"]
             del fe["rd"]
             if not icur:
-                break
+                continue
 
             if vn != dbv:
                 _, rd = vn.get_dbv(rd)
 
-            q = "select w from up where rd = ? and fn = ?"
-            r = None
+            q = "select mt.k, mt.v from up inner join mt on mt.w = substr(up.w,1,16) where up.rd = ? and up.fn = ? and +mt.k != 'x'"
             try:
-                r = icur.execute(q, (rd, fn)).fetchone()
+                r = icur.execute(q, (rd, fn))
             except Exception as ex:
                 if "database is locked" in str(ex):
                     break
 
                 try:
                     args = s3enc(idx.mem_cur, rd, fn)
-                    r = icur.execute(q, args).fetchone()
+                    r = icur.execute(q, args)
                 except:
-                    t = "tag list error, {}/{}\n{}"
+                    t = "tag read error, {}/{}\n{}"
                     self.log(t.format(rd, fn, min_ex()))
                     break
 
-            tags   = {}
-            fe["tags"] = tags
-
-            if not r:
-                continue
-
-            w = r[0][:16]
-            q = "select k, v from mt where w = ? and +k != 'x'"
-            try:
-                for k, v in icur.execute(q, (w,)):
-                    tagset.add(k)
-                    tags[k] = v
-            except:
-                t = "tag read error, {}/{} [{}]:\n{}"
-                self.log(t.format(rd, fn, w, min_ex()))
-                break
+            fe["tags"] = {k: v for k, v in r}
+            _ = [tagset.add(k) for k in fe["tags"]]
 
         if icur:
             taglist = [k for k in vn.flags.get("mte", "").split(",") if k in tagset]
