@@ -15,7 +15,7 @@ except:
     HAVE_SSL = False
 
 from . import util as Util
-from .__init__ import TYPE_CHECKING, E
+from .__init__ import TYPE_CHECKING, EnvParams
 from .authsrv import AuthSrv  # typechk
 from .httpcli import HttpCli
 from .ico import Ico
@@ -23,6 +23,7 @@ from .mtag import HAVE_FFMPEG
 from .th_cli import ThumbCli
 from .th_srv import HAVE_PIL, HAVE_VIPS
 from .u2idx import U2idx
+from .util import HMaccas, shut_socket
 
 try:
     raise Exception()
@@ -49,9 +50,11 @@ class HttpConn(object):
 
         self.mutex  = hsrv.mutex  # mypy404
         self.args  = hsrv.args  # mypy404
+        self.E  = self.args.E
         self.asrv  = hsrv.asrv  # mypy404
         self.cert_path = hsrv.cert_path
         self.u2fh  = hsrv.u2fh  # mypy404
+        self.iphash  = hsrv.broker.iphash
 
         enth = (HAVE_PIL or HAVE_VIPS or HAVE_FFMPEG) and not self.args.no_thumb
         self.thumbcli  = ThumbCli(hsrv) if enth else None  # mypy404
@@ -72,8 +75,7 @@ class HttpConn(object):
     def shutdown(self)  :
         self.stopping = True
         try:
-            self.s.shutdown(socket.SHUT_RDWR)
-            self.s.close()
+            shut_socket(self.log, self.s, 1)
         except:
             pass
 
@@ -91,7 +93,7 @@ class HttpConn(object):
         return self.log_src
 
     def respath(self, res_name )  :
-        return os.path.join(E.mod, "web", res_name)
+        return os.path.join(self.E.mod, "web", res_name)
 
     def log(self, msg , c   = 0)  :
         self.log_func(self.log_src, msg, c)
@@ -189,11 +191,7 @@ class HttpConn(object):
             except Exception as ex:
                 em = str(ex)
 
-                if "ALERT_BAD_CERTIFICATE" in em:
-                    # firefox-linux if there is no exception yet
-                    self.log("client rejected our certificate (nice)")
-
-                elif "ALERT_CERTIFICATE_UNKNOWN" in em:
+                if "ALERT_CERTIFICATE_UNKNOWN" in em:
                     # android-chrome keeps doing this
                     pass
 
