@@ -24,7 +24,7 @@ try:
 except:
     pass
 
-from .__init__ import ANYWIN, MACOS, PY2, VT100, WINDOWS, E, unicode
+from .__init__ import ANYWIN, MACOS, PY2, VT100, WINDOWS, EnvParams, unicode
 from .authsrv import AuthSrv
 from .mtag import HAVE_FFMPEG, HAVE_FFPROBE
 from .tcpsrv import TcpSrv
@@ -32,6 +32,7 @@ from .th_srv import HAVE_PIL, HAVE_VIPS, HAVE_WEBP, ThumbSrv
 from .up2k import Up2k
 from .util import (
     VERSIONS,
+    HMaccas,
     alltrace,
     ansi_re,
     min_ex,
@@ -55,6 +56,7 @@ class SvcHub(object):
     def __init__(self, args , argv , printed )  :
         self.args = args
         self.argv = argv
+        self.E  = args.E
         self.logf  = None
         self.logf_base_fn = ""
         self.stop_req = False
@@ -71,6 +73,8 @@ class SvcHub(object):
         self.next_day = 0
         self.tstack = 0.0
 
+        self.iphash = HMaccas(os.path.join(self.E.cfg, "iphash"), 8)
+
         if args.sss or args.s >= 3:
             args.ss = True
             args.lo = args.lo or "cpp-%Y-%m%d-%H%M%S.txt.xz"
@@ -85,6 +89,7 @@ class SvcHub(object):
             args.no_mv = True
             args.hardlink = True
             args.vague_403 = True
+            args.ban_404 = "50,60,1440"
             args.nih = True
 
         if args.s:
@@ -146,8 +151,8 @@ class SvcHub(object):
         self.args.th_dec = list(decs.keys())
         self.thumbsrv = None
         if not args.no_thumb:
-            t = "decoder preference: {}".format(", ".join(self.args.th_dec))
-            self.log("thumb", t)
+            t = ", ".join(self.args.th_dec) or "(None available)"
+            self.log("thumb", "decoder preference: {}".format(t))
 
             if "pil" in self.args.th_dec and not HAVE_WEBP:
                 msg = "disabling webp thumbnails because either libwebp is not available or your Pillow is too old"
@@ -206,6 +211,9 @@ class SvcHub(object):
         self.log("root", t, 1)
 
         self.retcode = 1
+        self.sigterm()
+
+    def sigterm(self)  :
         os.kill(os.getpid(), signal.SIGTERM)
 
     def cb_httpsrv_up(self)  :
@@ -261,7 +269,7 @@ class SvcHub(object):
 
         msg = "[+] opened logfile [{}]\n".format(fn)
         printed += msg
-        lh.write("t0: {:.3f}\nargv: {}\n\n{}".format(E.t0, " ".join(argv), printed))
+        lh.write("t0: {:.3f}\nargv: {}\n\n{}".format(self.E.t0, " ".join(argv), printed))
         self.logf = lh
         self.logf_base_fn = base_fn
         print(msg, end="")
@@ -269,7 +277,7 @@ class SvcHub(object):
     def run(self)  :
         self.tcpsrv.run()
 
-        thr = threading.Thread(target=self.thr_httpsrv_up)
+        thr = threading.Thread(target=self.thr_httpsrv_up, name="sig-hsrv-up2")
         thr.daemon = True
         thr.start()
 
@@ -308,7 +316,7 @@ class SvcHub(object):
             return "cannot reload; already in progress"
 
         self.reloading = True
-        t = threading.Thread(target=self._reload)
+        t = threading.Thread(target=self._reload, name="reloading")
         t.daemon = True
         t.start()
         return "reload initiated"
@@ -498,7 +506,10 @@ class SvcHub(object):
         if self.args.j == 1:
             return False
 
-        if mp.cpu_count() <= 1:
+        try:
+            if mp.cpu_count() <= 1:
+                raise Exception()
+        except:
             self.log("svchub", "only one CPU detected; multiprocessing disabled")
             return False
 
