@@ -16,6 +16,7 @@ function goto_up2k() {
 // usually it's undefined but some chromes throw on invoke
 var up2k = null,
     up2k_hooks = [],
+    hws = [],
     sha_js = window.WebAssembly ? 'hw' : 'ac',  // ff53,c57,sa11
     m = 'will use ' + sha_js + ' instead of native sha512 due to';
 
@@ -266,6 +267,9 @@ function U2pvis(act, btns, uc, st) {
         fo.cb[nchunk] = cbd;
         fo.bd += delta;
 
+        if (!fo.bd)
+            return;
+
         var p = r.perc(fo.bd, fo.bd0, fo.bt, fobj.t_uploading);
         fo.hp = f2f(p[0], 2) + '%, ' + p[1] + ', ' + f2f(p[2], 2) + ' MB/s';
 
@@ -611,6 +615,9 @@ function Donut(uc, st) {
     var r = this,
         el = null,
         psvg = null,
+        tenstrobe = null,
+        tstrober = null,
+        strobes = [],
         o = 20 * 2 * Math.PI,
         optab = QS('#ops a[data-dest="up2k"]');
 
@@ -660,11 +667,15 @@ function Donut(uc, st) {
         r.base = pos();
         optab.innerHTML = ya ? svg() : optab.getAttribute('ico');
         el = QS('#ops a .donut');
+        clearTimeout(tenstrobe);
         if (!ya) {
             favico.upd();
             wintitle();
+            if (document.visibilityState == 'hidden')
+                tenstrobe = setTimeout(enstrobe, 500); //debounce
         }
     };
+
     r.do = function () {
         if (!el)
             return;
@@ -697,6 +708,54 @@ function Donut(uc, st) {
             r.fc = 0;
         }
     };
+
+    function enstrobe() {
+        strobes = ['████████████████', '________________', '████████████████'];
+        tstrober = setInterval(strobe, 300);
+
+        if (uc.upsfx && actx && actx.state != 'suspended')
+            sfx();
+
+        // firefox may forget that filedrops are user-gestures so it can skip this:
+        if (uc.upnag && window.Notification && Notification.permission == 'granted')
+            new Notification(uc.nagtxt);
+    }
+
+    function strobe() {
+        var txt = strobes.pop();
+        wintitle(txt);
+        if (!txt)
+            clearInterval(tstrober);
+    }
+
+    function sfx() {
+        var osc = actx.createOscillator(),
+            gain = actx.createGain(),
+            gg = gain.gain,
+            ft = [660, 880, 440, 660, 880],
+            ofs = 0;
+
+        osc.connect(gain);
+        gain.connect(actx.destination);
+        var ct = actx.currentTime + 0.03;
+
+        osc.type = 'triangle';
+        while (ft.length)
+            osc.frequency.setTargetAtTime(
+                ft.shift(), ct + (ofs += 0.05), 0.001);
+
+        gg.value = 0.15;
+        gg.setTargetAtTime(0.8, ct, 0.01);
+        gg.setTargetAtTime(0.3, ct + 0.13, 0.01);
+        gg.setTargetAtTime(0, ct + ofs + 0.05, 0.02);
+
+        osc.start();
+        setTimeout(function () {
+            osc.stop();
+            osc.disconnect();
+            gain.disconnect();
+        }, 500);
+    }
 }
 
 
@@ -718,6 +777,11 @@ function up2k_init(subtle) {
         "gotallfiles": [gotallfiles]  // hooks
     };
 
+    setTimeout(function () {
+        if (window.WebAssembly && !hws.length)
+            fetch('/.cpr/w.hash.js' + CB);
+    }, 1000);
+
     function showmodal(msg) {
         ebi('u2notbtn').innerHTML = msg;
         ebi('u2btn').style.display = 'none';
@@ -747,7 +811,7 @@ function up2k_init(subtle) {
             showmodal('<h1>loading ' + fn + '</h1>');
             import_js('/.cpr/deps/' + fn, unmodal);
 
-            if (is_https) {
+            if (HTTPS) {
                 // chrome<37 firefox<34 edge<12 opera<24 safari<7
                 m = L.u_ancient;
                 setmsg('');
@@ -790,17 +854,20 @@ function up2k_init(subtle) {
     var parallel_uploads = icfg_get('nthread'),
         uc = {},
         fdom_ctr = 0,
-        min_filebuf = 0,
         biggest_file = 0;
 
     bcfg_bind(uc, 'multitask', 'multitask', true, null, false);
     bcfg_bind(uc, 'potato', 'potato', false, set_potato, false);
     bcfg_bind(uc, 'ask_up', 'ask_up', true, null, false);
-    bcfg_bind(uc, 'flag_en', 'flag_en', false, apply_flag_cfg);
     bcfg_bind(uc, 'fsearch', 'fsearch', false, set_fsearch, false);
-    bcfg_bind(uc, 'turbo', 'u2turbo', turbolvl > 1, draw_turbo, false);
-    bcfg_bind(uc, 'datechk', 'u2tdate', turbolvl < 3, null, false);
-    bcfg_bind(uc, 'az', 'u2sort', u2sort.indexOf('n') + 1, set_u2sort, false);
+
+    bcfg_bind(uc, 'flag_en', 'flag_en', false, apply_flag_cfg);
+    bcfg_bind(uc, 'turbo', 'u2turbo', turbolvl > 1, draw_turbo);
+    bcfg_bind(uc, 'datechk', 'u2tdate', turbolvl < 3, null);
+    bcfg_bind(uc, 'az', 'u2sort', u2sort.indexOf('n') + 1, set_u2sort);
+    bcfg_bind(uc, 'hashw', 'hashw', !!window.WebAssembly && (!subtle || !CHROME || MOBILE), set_hashw);
+    bcfg_bind(uc, 'upnag', 'upnag', false, set_upnag);
+    bcfg_bind(uc, 'upsfx', 'upsfx', false);
 
     var st = {
         "files": [],
@@ -838,6 +905,8 @@ function up2k_init(subtle) {
             "t": ""
         },
         "car": 0,
+        "slow_io": null,
+        "oserr": false,
         "modn": 0,
         "modv": 0,
         "mod0": null
@@ -859,11 +928,7 @@ function up2k_init(subtle) {
     r.st = st;
     r.uc = uc;
 
-    var bobslice = null;
-    if (window.File)
-        bobslice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice;
-
-    if (!bobslice || !window.FileReader || !window.FileList)
+    if (!window.File || !File.prototype.slice || !window.FileReader || !window.FileList)
         return un2k(L.u_ever);
 
     var flag = false;
@@ -871,7 +936,17 @@ function up2k_init(subtle) {
     set_fsearch();
 
     function nav() {
-        ebi('file' + fdom_ctr).click();
+        start_actx();
+
+        // too buggy on chrome <= 72
+        var m = / Chrome\/([0-9]+)\./.exec(navigator.userAgent);
+        if (m && parseInt(m[1]) < 73)
+            return ebi('file' + fdom_ctr).click();
+
+        modal.confirm(L.u_nav_m,
+            function () { ebi('file' + fdom_ctr).click(); },
+            function () { ebi('dir' + fdom_ctr).click(); },
+            null, L.u_nav_b);
     }
     ebi('u2btn').onclick = nav;
 
@@ -964,6 +1039,36 @@ function up2k_init(subtle) {
     }
     ebi('drops').onclick = offdrag;  // old ff
 
+    function gotdir(e) {
+        ev(e);
+        var good_files = [],
+            nil_files = [],
+            bad_files = [];
+
+        for (var a = 0, aa = e.target.files.length; a < aa; a++) {
+            var fobj = e.target.files[a],
+                name = fobj.webkitRelativePath,
+                dst = good_files;
+
+            try {
+                if (!name)
+                    throw 1;
+
+                if (fobj.size < 1)
+                    dst = nil_files;
+            }
+            catch (ex) {
+                dst = bad_files;
+            }
+            dst.push([fobj, name]);
+        }
+
+        if (!good_files.length && bad_files.length)
+            return toast.err(30, "that's not a folder!\n\nyour browser is too old,\nplease try dragdrop instead");
+
+        return read_dirs(null, [], [], good_files, nil_files, bad_files);
+    }
+
     function gotfile(e) {
         ev(e);
         nenters = 0;
@@ -1033,9 +1138,8 @@ function up2k_init(subtle) {
             }
             dst.push([fobj, fobj.name]);
         }
-        if (dirs) {
-            return read_dirs(null, [], dirs, good_files, nil_files, bad_files);
-        }
+        start_actx();  // good enough for chrome; not firefox
+        return read_dirs(null, [], dirs, good_files, nil_files, bad_files);
     }
 
     function rd_flatten(pf, dirs) {
@@ -1140,6 +1244,7 @@ function up2k_init(subtle) {
 
             msg += L.u_just1;
             return modal.alert(msg, function () {
+                start_actx();
                 gotallfiles(good_files, nil_files, []);
             });
         }
@@ -1151,8 +1256,10 @@ function up2k_init(subtle) {
 
             msg += L.u_just1;
             return modal.confirm(msg, function () {
+                start_actx();
                 gotallfiles(good_files.concat(nil_files), [], []);
             }, function () {
+                start_actx();
                 gotallfiles(good_files, [], []);
             });
         }
@@ -1163,23 +1270,40 @@ function up2k_init(subtle) {
             return a < b ? -1 : a > b ? 1 : 0;
         });
 
-        var msg = [L.u_asku.format(good_files.length, esc(get_vpath())) + '<ul>'];
+        var msg = [];
+
+        if (lifetime)
+            msg.push('<b>' + L.u_up_life.format(lhumantime(st.lifetime || lifetime)) + '</b>\n\n');
+
+        if (FIREFOX && good_files.length > 3000)
+            msg.push(L.u_ff_many + "\n\n");
+
+        msg.push(L.u_asku.format(good_files.length, esc(get_vpath())) + '<ul>');
         for (var a = 0, aa = Math.min(20, good_files.length); a < aa; a++)
             msg.push('<li>' + esc(good_files[a][1]) + '</li>');
 
         if (uc.ask_up && !uc.fsearch)
             return modal.confirm(msg.join('') + '</ul>', function () {
+                start_actx();
                 up_them(good_files);
-                toast.inf(15, L.u_unpt);
+                toast.inf(15, L.u_unpt, L.u_unpt);
             }, null);
 
         up_them(good_files);
     }
 
     function up_them(good_files) {
+        start_actx();
         var evpath = get_evpath(),
             draw_each = good_files.length < 50;
 
+        if (window.WebAssembly && !hws.length) {
+            for (var a = 0; a < Math.min(navigator.hardwareConcurrency || 4, 16); a++)
+                hws.push(new Worker('/.cpr/w.hash.js' + CB));
+
+            console.log(hws.length + " hashers");
+        }
+
         if (!uc.az)
             good_files.sort(function (a, b) {
                 a = a[0].size;
@@ -1239,9 +1363,7 @@ function up2k_init(subtle) {
 
             st.bytes.total += entry.size;
             st.files.push(entry);
-            if (!entry.size)
-                push_t(st.todo.handshake, entry);
-            else if (uc.turbo)
+            if (uc.turbo)
                 push_t(st.todo.head, entry);
             else
                 push_t(st.todo.hash, entry);
@@ -1251,14 +1373,25 @@ function up2k_init(subtle) {
             pvis.changecard(pvis.act);
         }
         ebi('u2tabw').className = 'ye';
+
+        setTimeout(function () {
+            if (!actx || actx.state != 'suspended' || toast.tag == L.u_unpt)
+                return;
+
+            toast.warn(30, "<div onclick=\"start_actx();toast.inf(3,'thanks!')\">please click this text to<br />unlock full upload speed</div>");
+        }, 500);
     }
 
     function more_one_file() {
         fdom_ctr++;
         var elm = mknod('div');
-        elm.innerHTML = '<input id="file{0}" type="file" name="file{0}[]" multiple="multiple" tabindex="-1" />'.format(fdom_ctr);
+        elm.innerHTML = (
+            '<input id="file{0}" type="file" name="file{0}[]" multiple="multiple" tabindex="-1" />' +
+            '<input id="dir{0}" type="file" name="dir{0}[]" multiple="multiple" tabindex="-1" webkitdirectory />'
+        ).format(fdom_ctr);
         ebi('u2form').appendChild(elm);
         ebi('file' + fdom_ctr).onchange = gotfile;
+        ebi('dir' + fdom_ctr).onchange = gotdir;
     }
     more_one_file();
 
@@ -1288,8 +1421,13 @@ function up2k_init(subtle) {
 
         if (!nhash) {
             var h = L.u_etadone.format(humansize(st.bytes.hashed), pvis.ctr.ok + pvis.ctr.ng);
-            if (st.eta.h !== h)
+            if (st.eta.h !== h) {
                 st.eta.h = ebi('u2etah').innerHTML = h;
+                console.log('{0} hash, {1} up, {2} busy'.format(
+                    f2f(st.time.hashing, 1),
+                    f2f(st.time.uploading, 1),
+                    f2f(st.time.busy, 1)));
+            }
         }
 
         if (!nsend && !nhash) {
@@ -1351,6 +1489,15 @@ function up2k_init(subtle) {
             etaskip = 0;
     }
 
+    function got_oserr() {
+        if (!hws.length || !uc.hashw || st.oserr)
+            return;
+
+        st.oserr = true;
+        var msg = HTTPS ? L.u_emtleak3 : L.u_emtleak2.format((window.location + '').replace(':', 's:'));
+        modal.alert(L.u_emtleak1 + msg + L.u_emtleak4 + (CHROME ? L.u_emtleakc : FIREFOX ? L.u_emtleakf : ''));
+    }
+
     /////
     ////
     ///   actuator
@@ -1455,6 +1602,7 @@ function up2k_init(subtle) {
 
                     if (!is_busy) {
                         uptoast();
+                        //throw console.hist.join('\n');
                     }
                     else {
                         timer.add(donut.do);
@@ -1574,15 +1722,15 @@ function up2k_init(subtle) {
         console.log('toast', ok, ng);
 
         if (ok && ng)
-            toast.warn(t, (sr ? L.ur_sm : L.ur_um).format(ok, ng));
+            toast.warn(t, uc.nagtxt = (sr ? L.ur_sm : L.ur_um).format(ok, ng));
         else if (ok > 1)
-            toast.ok(t, (sr ? L.ur_aso : L.ur_auo).format(ok));
+            toast.ok(t, uc.nagtxt = (sr ? L.ur_aso : L.ur_auo).format(ok));
         else if (ok)
-            toast.ok(t, sr ? L.ur_1so : L.ur_1uo);
+            toast.ok(t, uc.nagtxt = sr ? L.ur_1so : L.ur_1uo);
         else if (ng > 1)
-            toast.err(t, (sr ? L.ur_asn : L.ur_aun).format(ng));
+            toast.err(t, uc.nagtxt = (sr ? L.ur_asn : L.ur_aun).format(ng));
         else if (ng)
-            toast.err(t, sr ? L.ur_1sn : L.ur_1un);
+            toast.err(t, uc.nagtxt = sr ? L.ur_1sn : L.ur_1un);
 
         timer.rm(etafun);
         timer.rm(donut.do);
@@ -1663,8 +1811,12 @@ function up2k_init(subtle) {
 
     function exec_hash() {
         var t = st.todo.hash.shift();
+        if (!t.size)
+            return st.todo.handshake.push(t);
+
         st.busy.hash.push(t);
         st.nfile.hash = t.n;
+        t.t_hashing = Date.now();
 
         var bpend = 0,
             nchunk = 0,
@@ -1675,30 +1827,24 @@ function up2k_init(subtle) {
         pvis.setab(t.n, nchunks);
         pvis.move(t.n, 'bz');
 
+        if (hws.length && uc.hashw && (nchunks > 1 || document.visibilityState == 'hidden'))
+            // resolving subtle.digest w/o worker takes 1sec on blur if the actx hack breaks
+            return wexec_hash(t, chunksize, nchunks);
+
         var segm_next = function () {
-            if (nchunk >= nchunks || (bpend > chunksize && bpend >= min_filebuf))
+            if (nchunk >= nchunks || bpend)
                 return false;
 
             var reader = new FileReader(),
                 nch = nchunk++,
                 car = nch * chunksize,
-                cdr = car + chunksize,
-                t0 = Date.now();
+                cdr = Math.min(chunksize + car, t.size);
 
-            if (cdr >= t.size)
-                cdr = t.size;
-
-            bpend += cdr - car;
             st.bytes.hashed += cdr - car;
 
             function orz(e) {
-                if (!min_filebuf && nch == 1) {
-                    min_filebuf = 1;
-                    var td = Date.now() - t0;
-                    if (td > 50) {
-                        min_filebuf = 32 * 1024 * 1024;
-                    }
-                }
+                bpend--;
+                segm_next();
                 hash_calc(nch, e.target.result);
             }
             reader.onload = function (e) {
@@ -1715,6 +1861,7 @@ function up2k_init(subtle) {
                     pvis.seth(t.n, 2, err + ' @ ' + car);
                     console.log('OS-error', reader.error, '@', car);
                     handled = true;
+                    got_oserr();
                 }
 
                 if (handled) {
@@ -1726,15 +1873,13 @@ function up2k_init(subtle) {
 
                 toast.err(0, 'y o u   b r o k e    i t\nfile: ' + esc(t.name + '') + '\nerror: ' + err);
             };
-            reader.readAsArrayBuffer(
-                bobslice.call(t.fobj, car, cdr));
+            bpend++;
+            reader.readAsArrayBuffer(t.fobj.slice(car, cdr));
 
             return true;
         };
 
         var hash_calc = function (nch, buf) {
-            while (segm_next());
-
             var orz = function (hashbuf) {
                 var hslice = new Uint8Array(hashbuf).subarray(0, 33),
                     b64str = buf2b64(hslice);
@@ -1742,15 +1887,12 @@ function up2k_init(subtle) {
                 hashtab[nch] = b64str;
                 t.hash.push(nch);
                 pvis.hashed(t);
-
-                bpend -= buf.byteLength;
-                if (t.hash.length < nchunks) {
+                if (t.hash.length < nchunks)
                     return segm_next();
-                }
+
                 t.hash = [];
-                for (var a = 0; a < nchunks; a++) {
+                for (var a = 0; a < nchunks; a++)
                     t.hash.push(hashtab[a]);
-                }
 
                 t.t_hashed = Date.now();
 
@@ -1782,11 +1924,119 @@ function up2k_init(subtle) {
                 }
             }, 1);
         };
-
-        t.t_hashing = Date.now();
         segm_next();
     }
 
+    function wexec_hash(t, chunksize, nchunks) {
+        var nchunk = 0,
+            reading = 0,
+            max_readers = 1,
+            opt_readers = 2,
+            free = [],
+            busy = {},
+            nbusy = 0,
+            hashtab = {},
+            mem = (MOBILE ? 128 : 256) * 1024 * 1024;
+
+        for (var a = 0; a < hws.length; a++) {
+            var w = hws[a];
+            free.push(w);
+            w.onmessage = onmsg;
+            mem -= chunksize;
+            if (mem <= 0)
+                break;
+        }
+
+        function go_next() {
+            if (st.slow_io && uc.multitask)
+                // android-chrome filereader latency is ridiculous but scales linearly
+                // (unlike every other platform which instead suffers on parallel reads...)
+                max_readers = opt_readers = free.length;
+
+            if (reading >= max_readers || !free.length || nchunk >= nchunks)
+                return;
+
+            var w = free.pop(),
+                car = nchunk * chunksize,
+                cdr = Math.min(chunksize + car, t.size);
+
+            //console.log('[P ] %d read bgin (%d reading, %d busy)', nchunk, reading + 1, nbusy + 1);
+            w.postMessage([nchunk, t.fobj, car, cdr]);
+            busy[nchunk] = w;
+            nbusy++;
+            reading++;
+            nchunk++;
+        }
+
+        function onmsg(d) {
+            d = d.data;
+            var k = d[0];
+
+            if (k == "panic")
+                return vis_exh(d[1], 'up2k.js', '', '', d[1]);
+
+            if (k == "fail") {
+                pvis.seth(t.n, 1, d[1]);
+                pvis.seth(t.n, 2, d[2]);
+                console.log(d[1], d[2]);
+                if (d[1] == 'OS-error')
+                    got_oserr();
+
+                pvis.move(t.n, 'ng');
+                apop(st.busy.hash, t);
+                st.bytes.finished += t.size;
+                return;
+            }
+
+            if (k == "ferr")
+                return toast.err(0, 'y o u   b r o k e    i t\nfile: ' + esc(t.name + '') + '\nerror: ' + d[1]);
+
+            if (k == "read") {
+                reading--;
+                if (MOBILE && CHROME && st.slow_io === null && d[1] == 1 && d[2] > 1024 * 512) {
+                    var spd = Math.floor(d[2] / d[3]);
+                    st.slow_io = spd < 40 * 1024;
+                    console.log('spd {0}, slow: {1}'.format(spd, st.slow_io));
+                }
+                //console.log('[P ] %d read DONE (%d reading, %d busy)', d[1], reading, nbusy);
+                return go_next();
+            }
+
+            if (k == "done") {
+                var nchunk = d[1],
+                    hslice = d[2],
+                    sz = d[3];
+
+                free.push(busy[nchunk]);
+                delete busy[nchunk];
+                nbusy--;
+
+                //console.log('[P ] %d HASH DONE (%d reading, %d busy)', nchunk, reading, nbusy);
+
+                hashtab[nchunk] = buf2b64(hslice);
+                st.bytes.hashed += sz;
+                t.hash.push(nchunk);
+                pvis.hashed(t);
+
+                if (t.hash.length < nchunks)
+                    return nbusy < opt_readers && go_next();
+
+                t.hash = [];
+                for (var a = 0; a < nchunks; a++)
+                    t.hash.push(hashtab[a]);
+
+                t.t_hashed = Date.now();
+
+                pvis.seth(t.n, 2, L.u_hashdone);
+                pvis.seth(t.n, 1, '📦 wait');
+                apop(st.busy.hash, t);
+                st.todo.handshake.push(t);
+                tasker();
+            }
+        }
+        go_next();
+    }
+
     /////
     ////
     ///   head
@@ -1798,7 +2048,10 @@ function up2k_init(subtle) {
 
         var xhr = new XMLHttpRequest();
         xhr.onerror = function () {
-            console.log('head onerror, retrying', t);
+            console.log('head onerror, retrying', t.name, t);
+            if (!toast.visible)
+                toast.warn(9.98, L.u_enethd + "\n\nfile: " + t.name, t);
+
             apop(st.busy.head, t);
             st.todo.head.unshift(t);
         };
@@ -1854,22 +2107,25 @@ function up2k_init(subtle) {
         t.t_busied = me;
 
         if (keepalive)
-            console.log("sending keepalive handshake", t);
+            console.log("sending keepalive handshake", t.name, t);
 
         var xhr = new XMLHttpRequest();
         xhr.onerror = function () {
             if (t.t_busied != me) {
-                console.log('zombie handshake onerror,', t);
+                console.log('zombie handshake onerror,', t.name, t);
                 return;
             }
-            console.log('handshake onerror, retrying', t);
+            if (!toast.visible)
+                toast.warn(9.98, L.u_eneths + "\n\nfile: " + t.name, t);
+
+            console.log('handshake onerror, retrying', t.name, t);
             apop(st.busy.handshake, t);
             st.todo.handshake.unshift(t);
             t.keepalive = keepalive;
         };
         function orz(e) {
             if (t.t_busied != me) {
-                console.log('zombie handshake onload,', t);
+                console.log('zombie handshake onload,', t.name, t);
                 return;
             }
             if (xhr.status == 200) {
@@ -1879,6 +2135,9 @@ function up2k_init(subtle) {
                     return;
                 }
 
+                if (toast.tag === t)
+                    toast.ok(5, L.u_fixed);
+
                 var response = JSON.parse(xhr.responseText);
                 if (!response.name) {
                     var msg = '',
@@ -1964,7 +2223,7 @@ function up2k_init(subtle) {
                             'npart': t.postlist[a]
                         });
 
-                    msg = L.u_upping;
+                    msg = null;
                     done = false;
 
                     if (sort)
@@ -1974,7 +2233,10 @@ function up2k_init(subtle) {
                                     a.npart < b.npart ? -1 : 1;
                         });
                 }
-                pvis.seth(t.n, 1, msg);
+
+                if (msg)
+                    pvis.seth(t.n, 1, msg);
+
                 apop(st.busy.handshake, t);
 
                 if (done) {
@@ -1985,7 +2247,7 @@ function up2k_init(subtle) {
                         spd2 = (t.size / ((t.t_uploaded - t.t_uploading) / 1000.)) / (1024 * 1024.);
 
                     pvis.seth(t.n, 2, 'hash {0}, up {1} MB/s'.format(
-                        f2f(spd1, 2), isNaN(spd2) ? '--' : f2f(spd2, 2)));
+                        f2f(spd1, 2), !isNum(spd2) ? '--' : f2f(spd2, 2)));
 
                     pvis.move(t.n, 'ok');
                     if (!pvis.ctr.bz && !pvis.ctr.q)
@@ -2001,7 +2263,7 @@ function up2k_init(subtle) {
             }
             else {
                 pvis.seth(t.n, 1, "ERROR");
-                pvis.seth(t.n, 2, L.u_ehstmp);
+                pvis.seth(t.n, 2, L.u_ehstmp, t);
 
                 var err = "",
                     rsp = (xhr.responseText + ''),
@@ -2022,7 +2284,6 @@ function up2k_init(subtle) {
                     return;
                 }
 
-                st.bytes.finished += t.size;
                 var err_pend = rsp.indexOf('partial upload exists') + 1,
                     err_dupe = rsp.indexOf('file already exists') + 1;
 
@@ -2041,6 +2302,9 @@ function up2k_init(subtle) {
                     return toast.err(0, L.u_ehsdf + "\n\n" + rsp.replace(/.*; /, ''));
 
                 if (err != "") {
+                    if (!t.t_uploading)
+                        st.bytes.finished += t.size;
+
                     pvis.seth(t.n, 1, "ERROR");
                     pvis.seth(t.n, 2, err);
                     pvis.move(t.n, 'ng');
@@ -2050,7 +2314,7 @@ function up2k_init(subtle) {
                     return;
                 }
                 err = t.t_uploading ? L.u_ehsfin : t.srch ? L.u_ehssrch : L.u_ehsinit;
-                xhrchk(xhr, err + ";\n\nfile: " + t.name + "\n\nerror ", "404, target folder not found");
+                xhrchk(xhr, err + "\n\nfile: " + t.name + "\n\nerror ", "404, target folder not found", "warn", t);
             }
         }
         xhr.onload = function (e) {
@@ -2061,6 +2325,7 @@ function up2k_init(subtle) {
             "name": t.name,
             "size": t.size,
             "lmod": t.lmod,
+            "life": st.lifetime,
             "hash": t.hash
         };
         if (t.srch)
@@ -2078,7 +2343,14 @@ function up2k_init(subtle) {
 
     function can_upload_next() {
         var upt = st.todo.upload[0],
-            upf = st.files[upt.nfile];
+            upf = st.files[upt.nfile],
+            now = Date.now();
+
+        for (var a = 0, aa = st.busy.handshake.length; a < aa; a++) {
+            var hs = st.busy.handshake[a];
+            if (hs.n < upt.nfile && hs.t_busied > now - 10 * 1000 && !st.files[hs.n].bytes_uploaded)
+                return false;  // handshake race; wait for lexically first
+        }
 
         if (upf.sprs)
             return true;
@@ -2121,10 +2393,10 @@ function up2k_init(subtle) {
             }
             else if (txt.indexOf('already got that') + 1 ||
                 txt.indexOf('already being written') + 1) {
-                console.log("ignoring dupe-segment error", t);
+                console.log("ignoring dupe-segment error", t.name, t);
             }
             else {
-                xhrchk(xhr, L.u_cuerr2.format(npart, Math.ceil(t.size / chunksize), t.name), "404, target folder not found (???)");
+                xhrchk(xhr, L.u_cuerr2.format(npart, Math.ceil(t.size / chunksize), t.name), "404, target folder not found (???)", "warn", t);
 
                 chill(t);
             }
@@ -2156,9 +2428,9 @@ function up2k_init(subtle) {
                     return;
 
                 if (!toast.visible)
-                    toast.warn(9.98, L.u_cuerr.format(npart, Math.ceil(t.size / chunksize), t.name));
+                    toast.warn(9.98, L.u_cuerr.format(npart, Math.ceil(t.size / chunksize), t.name), t);
 
-                console.log('chunkpit onerror,', ++tries, t);
+                console.log('chunkpit onerror,', ++tries, t.name, t);
                 orz2(xhr);
             };
             xhr.open('POST', t.purl, true);
@@ -2172,7 +2444,7 @@ function up2k_init(subtle) {
                 xhr.overrideMimeType('Content-Type', 'application/octet-stream');
 
             xhr.responseType = 'text';
-            xhr.send(bobslice.call(t.fobj, car, cdr));
+            xhr.send(t.fobj.slice(car, cdr));
         }
         do_send();
     }
@@ -2188,9 +2460,8 @@ function up2k_init(subtle) {
             wpx = window.innerWidth,
             fpx = parseInt(getComputedStyle(bar)['font-size']),
             wem = wpx * 1.0 / fpx,
-            write = has(perms, 'write'),
-            wide = write && wem > 54 ? 'w' : '',
-            parent = ebi(wide && write ? 'u2btn_cw' : 'u2btn_ct'),
+            wide = wem > 54 ? 'w' : '',
+            parent = ebi(wide ? 'u2btn_cw' : 'u2btn_ct'),
             btn = ebi('u2btn');
 
         if (btn.parentNode !== parent) {
@@ -2198,8 +2469,8 @@ function up2k_init(subtle) {
             ebi('u2conf').className = ebi('u2cards').className = ebi('u2etaw').className = wide;
         }
 
-        wide = write && wem > 82 ? 'ww' : wide;
-        parent = ebi(wide == 'ww' && write ? 'u2c3w' : 'u2c3t');
+        wide = wem > 82 ? 'ww' : wide;
+        parent = ebi(wide == 'ww' ? 'u2c3w' : 'u2c3t');
         var its = [ebi('u2etaw'), ebi('u2cards')];
         if (its[0].parentNode !== parent) {
             ebi('u2conf').className = wide;
@@ -2212,7 +2483,7 @@ function up2k_init(subtle) {
     window.addEventListener('resize', onresize);
     onresize();
 
-    if (is_touch) {
+    if (MOBILE) {
         // android-chrome wobbles for a bit; firefox / iOS-safari are OK
         setTimeout(onresize, 20);
         setTimeout(onresize, 100);
@@ -2288,19 +2559,83 @@ function up2k_init(subtle) {
     }
     draw_turbo();
 
+    function draw_life() {
+        var el = ebi('u2life');
+        if (!lifetime) {
+            el.style.display = 'none';
+            el.innerHTML = '';
+            st.lifetime = 0;
+            return;
+        }
+        el.style.display = uc.fsearch ? 'none' : '';
+        el.innerHTML = '<div>' + L.u_life_cfg + '</div><div>' + L.u_life_est + '</div><div id="undor"></div>';
+        set_life(Math.min(lifetime, icfg_get('lifetime', lifetime)));
+        ebi('lifem').oninput = ebi('lifeh').oninput = mod_life;
+        tt.att(ebi('u2life'));
+    }
+    draw_life();
+
+    function mod_life(e) {
+        var el = e.target,
+            pow = parseInt(el.getAttribute('p')),
+            v = parseInt(el.value);
+
+        if (!isNum(v))
+            return;
+
+        if (toast.tag == mod_life)
+            toast.hide();
+
+        v *= pow;
+        if (v > lifetime) {
+            v = lifetime;
+            toast.warn(20, L.u_life_max.format(lhumantime(lifetime)), mod_life);
+        }
+
+        swrite('lifetime', v);
+        set_life(v);
+    }
+
+    function set_life(v) {
+        //ebi('lifes').value = v;
+        ebi('lifem').value = parseInt(v / 60);
+        ebi('lifeh').value = parseInt(v / 3600);
+
+        var undo = have_unpost - (v || lifetime);
+        ebi('undor').innerHTML = undo <= 0 ?
+            L.u_unp_ng : L.u_unp_ok.format(lhumantime(undo));
+
+        st.lifetime = v;
+        rel_life();
+    }
+
+    function rel_life() {
+        if (!lifetime)
+            return;
+
+        try {
+            ebi('lifew').innerHTML = unix2iso((st.lifetime || lifetime) +
+                Date.now() / 1000 - new Date().getTimezoneOffset() * 60
+            ).replace(' ', ', ').slice(0, -3);
+        }
+        catch (ex) { }
+    }
+    setInterval(rel_life, 9000);
+
     function set_potato() {
         pvis.potato();
         set_fsearch();
     }
 
     function set_fsearch(new_state) {
-        var fixed = false;
+        var fixed = false,
+            can_write = false;
 
         if (!ebi('fsearch')) {
             new_state = false;
         }
         else if (perms.length) {
-            if (!has(perms, 'write')) {
+            if (!(can_write = has(perms, 'write'))) {
                 new_state = true;
                 fixed = true;
             }
@@ -2310,12 +2645,11 @@ function up2k_init(subtle) {
             }
         }
 
-        if (new_state !== undefined) {
-            uc.fsearch = new_state;
-            bcfg_set('fsearch', uc.fsearch);
-        }
+        if (new_state !== undefined)
+            bcfg_set('fsearch', uc.fsearch = new_state);
 
         try {
+            clmod(ebi('u2c3w'), 's', !can_write);
             QS('label[for="fsearch"]').style.display = QS('#fsearch').style.display = fixed ? 'none' : '';
         }
         catch (ex) { }
@@ -2336,6 +2670,7 @@ function up2k_init(subtle) {
         ebi('u2mu').style.display = potato ? '' : 'none';
 
         draw_turbo();
+        draw_life();
         onresize();
     }
 
@@ -2346,7 +2681,7 @@ function up2k_init(subtle) {
             }
             catch (ex) {
                 toast.err(5, "not supported on your browser:\n" + esc(basenames(ex)));
-                bcfg_set('flag_en', false);
+                bcfg_set('flag_en', uc.flag_en = false);
             }
         }
         else if (!uc.flag_en && flag) {
@@ -2366,6 +2701,34 @@ function up2k_init(subtle) {
         localStorage.removeItem('u2sort');
     }
 
+    function set_hashw() {
+        if (!window.WebAssembly) {
+            bcfg_set('hashw', uc.hashw = false);
+            toast.err(10, L.u_nowork);
+        }
+    }
+
+    function set_upnag(en) {
+        function nopenag() {
+            bcfg_set('upnag', uc.upnag = false);
+            toast.err(10, "https only");
+        }
+
+        function chknag() {
+            if (Notification.permission != 'granted')
+                nopenag();
+        }
+
+        if (!window.Notification || !HTTPS)
+            return nopenag();
+
+        if (en && Notification.permission == 'default')
+            Notification.requestPermission().then(chknag, chknag);
+    }
+
+    if (uc.upnag && (!window.Notification || Notification.permission != 'granted'))
+        bcfg_set('upnag', uc.upnag = false);
+
     ebi('nthread_add').onclick = function (e) {
         ev(e);
         bumpthread(1);
