@@ -7,12 +7,30 @@ if (!window['console'])
 
 
 var wah = '',
+    CB = '?_=' + Date.now(),
     HALFMAX = 8192 * 8192 * 8192 * 8192,
-    is_touch = 'ontouchstart' in window,
-    is_https = (window.location + '').indexOf('https:') === 0,
-    IPHONE = is_touch && /iPhone|iPad|iPod/i.test(navigator.userAgent),
+    HTTPS = (window.location + '').indexOf('https:') === 0,
+    TOUCH = 'ontouchstart' in window,
+    MOBILE = TOUCH,
+    CHROME = !!window.chrome,
+    FIREFOX = ('netscape' in window) && / rv:/.test(navigator.userAgent),
+    IPHONE = TOUCH && /iPhone|iPad|iPod/i.test(navigator.userAgent),
     WINDOWS = navigator.platform ? navigator.platform == 'Win32' : /Windows/.test(navigator.userAgent);
 
+try {
+    CB = '?' + document.currentScript.src.split('?').pop();
+
+    if (navigator.userAgentData.mobile)
+        MOBILE = true;
+
+    if (navigator.userAgentData.platform == 'Windows')
+        WINDOWS = true;
+
+    if (navigator.userAgentData.brands.some(function (d) { return d.brand == 'Chromium' }))
+        CHROME = true;
+}
+catch (ex) { }
+
 
 var ebi = document.getElementById.bind(document),
     QS = document.querySelector.bind(document),
@@ -93,7 +111,7 @@ catch (ex) {
         console.log = console.stdlog;
     console.log('console capture failed', ex);
 }
-var crashed = false, ignexd = {};
+var crashed = false, ignexd = {}, evalex_fatal = false;
 function vis_exh(msg, url, lineNo, columnNo, error) {
     if ((msg + '').indexOf('ResizeObserver') + 1)
         return;  // chrome issue 809574 (benign, from <video>)
@@ -104,7 +122,7 @@ function vis_exh(msg, url, lineNo, columnNo, error) {
     if (!/\.js($|\?)/.exec('' + url))
         return;  // chrome debugger
 
-    if ((url + '').indexOf(' > eval') + 1)
+    if ((url + '').indexOf(' > eval') + 1 && !evalex_fatal)
         return;  // md timer
 
     var ekey = url + '\n' + lineNo + '\n' + msg;
@@ -117,7 +135,7 @@ function vis_exh(msg, url, lineNo, columnNo, error) {
         '<h1>you hit a bug!</h1>',
         '<p style="font-size:1.3em;margin:0">try to <a href="#" onclick="localStorage.clear();location.reload();">reset copyparty settings</a> if you are stuck here, or <a href="#" onclick="ignex();">ignore this</a> / <a href="#" onclick="ignex(true);">ignore all</a></p>',
         '<p style="color:#fff">please send me a screenshot arigathanks gozaimuch: <a href="<ghi>" target="_blank">github issue</a> or <code>ed#2644</code></p>',
-        '<p class="b">' + esc(url + ' @' + lineNo + ':' + columnNo), '<br />' + esc(String(msg)) + '</p>',
+        '<p class="b">' + esc(url + ' @' + lineNo + ':' + columnNo), '<br />' + esc(String(msg)).replace(/\n/g, '<br />') + '</p>',
         '<p><b>UA:</b> ' + esc(navigator.userAgent + '')
     ];
 
@@ -217,7 +235,7 @@ function ev(e) {
         return;
 
     if (e.preventDefault)
-        e.preventDefault()
+        e.preventDefault();
 
     if (e.stopPropagation)
         e.stopPropagation();
@@ -372,6 +390,21 @@ if (window.matchMedia) {
 }
 
 
+function yscroll() {
+    if (document.documentElement.scrollTop) {
+        return (window.yscroll = function () {
+            return document.documentElement.scrollTop;
+        })();
+    }
+    if (window.pageYOffset) {
+        return (window.yscroll = function () {
+            return window.pageYOffset;
+        })();
+    }
+    return 0;
+}
+
+
 function showsort(tab) {
     var v, vn, v1, v2, th = tab.tHead,
         sopts = jread('fsort', [["href", 1, ""]]);
@@ -511,7 +544,7 @@ function linksplit(rp, id) {
             link = rp.slice(0, ofs + 1);
             rp = rp.slice(ofs + 1);
         }
-        var vlink = esc(uricom_dec(link)[0]);
+        var vlink = esc(uricom_dec(link));
 
         if (link.indexOf('/') !== -1) {
             vlink = vlink.slice(0, -1) + '<span>/</span>';
@@ -569,6 +602,17 @@ function url_enc(txt) {
 
 
 function uricom_dec(txt) {
+    try {
+        return decodeURIComponent(txt);
+    }
+    catch (ex) {
+        console.log("ucd-err [" + txt + "]");
+        return txt;
+    }
+}
+
+
+function uricom_sdec(txt) {
     try {
         return [decodeURIComponent(txt), true];
     }
@@ -582,7 +626,7 @@ function uricom_dec(txt) {
 function uricom_adec(arr, li) {
     var ret = [];
     for (var a = 0; a < arr.length; a++) {
-        var txt = uricom_dec(arr[a])[0];
+        var txt = uricom_dec(arr[a]);
         ret.push(li ? '<li>' + esc(txt) + '</li>' : txt);
     }
 
@@ -604,7 +648,7 @@ function get_evpath() {
 
 
 function get_vpath() {
-    return uricom_dec(get_evpath())[0];
+    return uricom_dec(get_evpath());
 }
 
 
@@ -634,6 +678,14 @@ function s2ms(s) {
 }
 
 
+var isNum = function (v) {
+    var n = parseFloat(v);
+    return !isNaN(v - n) && n === v;
+};
+if (window.Number && Number.isFinite)
+    isNum = Number.isFinite;
+
+
 function f2f(val, nd) {
     // 10.toFixed(1) returns 10.00 for certain values of 10
     val = (val * Math.pow(10, nd)).toFixed(0).split('.')[0];
@@ -642,12 +694,13 @@ function f2f(val, nd) {
 
 
 function humansize(b, terse) {
-    var i = 0, u = terse ? ['B', 'K', 'M', 'G'] : ['B', 'KB', 'MB', 'GB'];
-    while (b >= 1000 && i < u.length) {
+    var i = 0, u = ['B', 'KB', 'MB', 'GB', 'TB', 'PB'];
+    while (b >= 1000 && i < u.length - 1) {
         b /= 1024;
         i += 1;
     }
-    return f2f(b, b >= 100 ? 0 : b >= 10 ? 1 : 2) + ' ' + u[i];
+    return (f2f(b, b >= 100 ? 0 : b >= 10 ? 1 : 2) +
+        ' ' + (terse ? u[i].charAt(0) : u[i]));
 }
 
 
@@ -664,7 +717,7 @@ function humantime(v) {
 }
 
 
-function shumantime(v) {
+function shumantime(v, long) {
     if (v < 10)
         return f2f(v, 2) + 's';
     if (v < 60)
@@ -684,11 +737,27 @@ function shumantime(v) {
         var v1 = parseInt(v / m1),
             v2 = ('0' + parseInt((v % m1) / m2)).slice(-2);
 
-        return v1 + ch + (v1 >= 10 ? '' : v2);
+        return v1 + ch + (v1 >= 10 || v2 == '00' ? '' : v2 + (
+            long && a < st.length - 1 ? st[a + 1][2] : ''));
     }
 }
 
 
+function lhumantime(v) {
+    var t = shumantime(v, 1),
+        tp = t.replace(/([a-z])/g, " $1 ").split(/ /g).slice(0, -1);
+
+    if (!window.L || tp.length < 2 || tp[1].indexOf('$') + 1)
+        return t;
+
+    var ret = '';
+    for (var a = 0; a < tp.length; a += 2)
+        ret += tp[a] + ' ' + L['ht_' + tp[a + 1]].replace(tp[a] == 1 ? /!.*/ : /!/, '') + L.ht_and;
+
+    return ret.slice(0, -L.ht_and.length);
+}
+
+
 function clamp(v, a, b) {
     return Math.min(Math.max(v, a), b);
 }
@@ -759,7 +828,7 @@ function fcfg_get(name, defval) {
     var o = ebi(name),
         val = parseFloat(sread(name));
 
-    if (isNaN(val))
+    if (!isNum(val))
         return parseFloat(o ? o.value : defval);
 
     if (o)
@@ -850,14 +919,18 @@ function scfg_bind(obj, oname, cname, defval, cb) {
 
 function hist_push(url) {
     console.log("h-push " + url);
-    if (window.history && history.pushState)
+    try {
         history.pushState(url, url, url);
+    }
+    catch (ex) { }
 }
 
 function hist_replace(url) {
     console.log("h-repl " + url);
-    if (window.history && history.replaceState)
+    try {
         history.replaceState(url, url, url);
+    }
+    catch (ex) { }  // ff "The operation is insecure." on rapid switches
 }
 
 function sethash(hv) {
@@ -936,7 +1009,7 @@ var tt = (function () {
         prev = this;
     };
 
-    var tev;
+    var tev, vh;
     r.dshow = function (e) {
         clearTimeout(tev);
         if (!r.getmsg(this))
@@ -946,9 +1019,10 @@ var tt = (function () {
             return r.show.bind(this)();
 
         tev = setTimeout(r.show.bind(this), 800);
-        if (is_touch)
+        if (TOUCH)
             return;
 
+        vh = window.innerHeight;
         this.addEventListener('mousemove', r.move);
         clmod(r.th, 'act', 1);
         r.move(e);
@@ -1010,7 +1084,7 @@ var tt = (function () {
     };
 
     r.hide = function (e) {
-        ev(e);
+        //ev(e);  // eats checkbox-label clicks
         clearTimeout(tev);
         window.removeEventListener('scroll', r.hide);
 
@@ -1027,8 +1101,9 @@ var tt = (function () {
     };
 
     r.move = function (e) {
+        var sy = e.clientY + 128 > vh ? -1 : 1;
         r.th.style.left = (e.pageX + 12) + 'px';
-        r.th.style.top = (e.pageY + 12) + 'px';
+        r.th.style.top = (e.pageY + 12 * sy) + 'px';
     };
 
     if (IPHONE) {
@@ -1098,6 +1173,7 @@ var toast = (function () {
     document.body.appendChild(obj);
     r.visible = false;
     r.txt = null;
+    r.tag = obj;  // filler value (null is scary)
 
     function scrollchk() {
         if (scrolling)
@@ -1126,9 +1202,10 @@ var toast = (function () {
         clearTimeout(te);
         clmod(obj, 'vis');
         r.visible = false;
+        r.tag = obj;
     };
 
-    r.show = function (cl, sec, txt) {
+    r.show = function (cl, sec, txt, tag) {
         clearTimeout(te);
         if (sec)
             te = setTimeout(r.hide, sec * 1000);
@@ -1144,19 +1221,20 @@ var toast = (function () {
         timer.add(scrollchk);
         r.visible = true;
         r.txt = txt;
+        r.tag = tag;
     };
 
-    r.ok = function (sec, txt) {
-        r.show('ok', sec, txt);
+    r.ok = function (sec, txt, tag) {
+        r.show('ok', sec, txt, tag);
     };
-    r.inf = function (sec, txt) {
-        r.show('inf', sec, txt);
+    r.inf = function (sec, txt, tag) {
+        r.show('inf', sec, txt, tag);
     };
-    r.warn = function (sec, txt) {
-        r.show('warn', sec, txt);
+    r.warn = function (sec, txt, tag) {
+        r.show('warn', sec, txt, tag);
     };
-    r.err = function (sec, txt) {
-        r.show('err', sec, txt);
+    r.err = function (sec, txt, tag) {
+        r.show('err', sec, txt, tag);
     };
 
     return r;
@@ -1170,9 +1248,16 @@ var modal = (function () {
         cb_up = null,
         cb_ok = null,
         cb_ng = null,
-        prim = '<a href="#" id="modal-ok">OK</a>',
-        sec = '<a href="#" id="modal-ng">Cancel</a>',
+        tok, tng, prim, sec, ok_cancel;
+
+    r.load = function () {
+        tok = (window.L && L.m_ok) || 'OK';
+        tng = (window.L && L.m_ng) || 'Cancel';
+        prim = '<a href="#" id="modal-ok">' + tok + '</a>';
+        sec = '<a href="#" id="modal-ng">' + tng + '</a>';
         ok_cancel = WINDOWS ? prim + sec : sec + prim;
+    };
+    r.load();
 
     r.busy = false;
 
@@ -1279,17 +1364,17 @@ var modal = (function () {
         r.show(html);
     }
 
-    r.confirm = function (html, cok, cng, fun) {
+    r.confirm = function (html, cok, cng, fun, btns) {
         q.push(function () {
-            _confirm(lf2br(html), cok, cng, fun);
+            _confirm(lf2br(html), cok, cng, fun, btns);
         });
         next();
     }
-    function _confirm(html, cok, cng, fun) {
+    function _confirm(html, cok, cng, fun, btns) {
         cb_ok = cok;
         cb_ng = cng === undefined ? cok : cng;
         cb_up = fun;
-        html += '<div id="modalb">' + ok_cancel + '</div>';
+        html += '<div id="modalb">' + (btns || ok_cancel) + '</div>';
         r.show(html);
     }
 
@@ -1391,8 +1476,10 @@ function repl(e) {
         if (!cmd)
             return toast.inf(3, 'eval aborted');
 
-        if (cmd.startsWith(','))
-            return modal.alert(esc(eval(cmd.slice(1)) + ''))
+        if (cmd.startsWith(',')) {
+            evalex_fatal = true;
+            return modal.alert(esc(eval(cmd.slice(1)) + ''));
+        }
 
         try {
             modal.alert(esc(eval(cmd) + ''));
@@ -1520,26 +1607,26 @@ var favico = (function () {
 
 
 var cf_cha_t = 0;
-function xhrchk(xhr, prefix, e404) {
+function xhrchk(xhr, prefix, e404, lvl, tag) {
     if (xhr.status < 400 && xhr.status >= 200)
         return true;
 
     if (xhr.status == 403)
-        return toast.err(0, prefix + (window.L && L.xhr403 || "403: access denied\n\ntry pressing F5, maybe you got logged out"));
+        return toast.err(0, prefix + (window.L && L.xhr403 || "403: access denied\n\ntry pressing F5, maybe you got logged out"), tag);
 
     if (xhr.status == 404)
-        return toast.err(0, prefix + e404);
+        return toast.err(0, prefix + e404, tag);
 
     var errtxt = (xhr.response && xhr.response.err) || xhr.responseText,
-        fun = toast.err;
+        fun = toast[lvl || 'err'];
 
-    if (xhr.status == 503 && /\bDD(?:wah){0}[o]S [Pp]rote[c]tion|>Just a mo[m]ent|#cf-b[u]bbles|Chec[k]ing your br[o]wser/.test(errtxt)) {
+    if (xhr.status == 503 && /[Cc]loud[f]lare|>Just a mo[m]ent|#cf-b[u]bbles|Chec[k]ing your br[o]wser/.test(errtxt)) {
         var now = Date.now(), td = now - cf_cha_t;
         if (td < 15000)
             return;
 
         cf_cha_t = now;
-        errtxt = 'Cloudflare DD' + wah + 'oS protection kicked in\n\n<strong>trying to fix it...</strong>';
+        errtxt = 'Clou' + wah + 'dflare protection kicked in\n\n<strong>trying to fix it...</strong>';
         fun = toast.warn;
 
         qsr('#cf_frame');
@@ -1548,5 +1635,5 @@ function xhrchk(xhr, prefix, e404) {
         document.body.appendChild(fr);
     }
 
-    return fun(0, prefix + xhr.status + ": " + errtxt);
+    return fun(0, prefix + xhr.status + ": " + errtxt, tag);
 }
